@@ -3,6 +3,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 import scipy.constants
 import scipy.sparse.linalg
+from scipy.constants import epsilon_0, speed_of_light
 from skfem import (
     Basis,
     BilinearForm,
@@ -27,12 +28,13 @@ def compute_modes(
     basis_epsilon_r,
     epsilon_r,
     wavelength,
-    mu_r,
-    num_modes,
+    mu_r=1,
+    num_modes=1,
     order=1,
     metallic_boundaries=False,
     radius=np.inf,
-    solver="scipy",
+    n_guess=None,
+    solver="slepc",
 ):
     if solver == "scipy":
         solver = solver_eigen_scipy
@@ -57,7 +59,7 @@ def compute_modes(
 
     @BilinearForm(dtype=epsilon_r.dtype)
     def aform(e_t, e_z, v_t, v_z, w):
-        epsilon = w.epsilon * (1 + w.x[0] / radius)
+        epsilon = w.epsilon * (1 + w.x[0] / radius) ** 2
 
         return (
             1 / mu_r * curl(e_t) * curl(v_t)
@@ -74,16 +76,21 @@ def compute_modes(
     A = aform.assemble(basis, epsilon=basis_epsilon_r.interpolate(epsilon_r))
     B = bform.assemble(basis, epsilon=basis_epsilon_r.interpolate(epsilon_r))
 
+    if n_guess:
+        sigma = sigma = k0**2 * n_guess**2
+    else:
+        sigma = sigma = k0**2 * np.max(epsilon_r) ** 2
+
     if metallic_boundaries:
         lams, xs = solve(
-            *condense(-A, -B, D=basis.get_dofs()),
-            solver=solver(k=num_modes, sigma=k0**2 * np.max(epsilon_r) ** 2),
+            *condense(-A, -B, D=basis.get_dofs(), x=basis.zeros(dtype=complex)),
+            solver=solver(k=num_modes, sigma=sigma),
         )
     else:
         lams, xs = solve(
             -A,
             -B,
-            solver=solver(k=num_modes, sigma=k0**2 * np.max(epsilon_r) ** 2),
+            solver=solver(k=num_modes, sigma=sigma),
         )
 
     idx = np.abs(np.real(lams)).argsort()[::-1]
@@ -145,7 +152,7 @@ def calculate_energy_current_density(basis, xs):
 
 
 def calculate_overlap(basis_i, E_i, H_i, basis_j, E_j, H_j):
-    @Functional
+    @Functional(dtype=np.complex64)
     def overlap(w):
         return cross(np.conj(w["E_i"][0]), w["H_j"][0]) + cross(w["E_j"][0], np.conj(w["H_i"][0]))
 
@@ -206,7 +213,7 @@ def calculate_scalar_product(basis_i, E_i, basis_j, H_j):
 
 
 def calculate_coupling_coefficient(basis_epsilon, delta_epsilon, basis, E_i, E_j):
-    @Functional
+    @Functional(dtype=complex)
     def overlap(w):
         return w["delta_epsilon"] * (
             dot(np.conj(w["E_i"][0]), w["E_j"][0]) + np.conj(w["E_i"][1]) * w["E_j"][1]
@@ -224,8 +231,12 @@ def confinement_factor(basis_epsilon, epsilon, basis, E):
     @Functional
     def factor(w):
         return (
-            np.sqrt(w["epsilon"]) * dot(np.conj(w["E"][0]), w["E"][0])
-            + np.conj(w["E"][1]) * w["E"][1]
+            (
+                np.sqrt(w["epsilon"])
+                * (dot(np.conj(w["E"][0]), w["E"][0]) + np.conj(w["E"][1]) * w["E"][1])
+            )
+            * speed_of_light
+            * epsilon_0
         )
 
     return factor.assemble(
@@ -365,11 +376,12 @@ if __name__ == "__main__":
     lams, basis, xs = compute_modes(
         basis0, epsilon, wavelength=1.55, mu_r=1, num_modes=6, order=2, radius=3
     )
-
     print(lams)
 
     plot_mode(basis, np.real(xs[0]))
     plt.show()
+    plot_mode(basis, np.real(xs[1]))
+    plt.show()
     plot_mode(basis, np.imag(xs[0]))
     plt.show()
 
@@ -385,8 +397,18 @@ if __name__ == "__main__":
         for j in range(len(lams)):
             E_i = xs[i]
             E_j = xs[j]
-            H_i = calculate_hfield(basis, E_i, lams[i] * (2 * np.pi / 1.55))
-            H_j = calculate_hfield(basis, E_j, lams[j] * (2 * np.pi / 1.55))
+            H_i = calculate_hfield(
+                basis,
+                E_i,
+                lams[i] * (2 * np.pi / 1.55),
+                omega=2 * np.pi / 1.55 * scipy.constants.speed_of_light,
+            )
+            H_j = calculate_hfield(
+                basis,
+                E_j,
+                lams[j] * (2 * np.pi / 1.55),
+                omega=2 * np.pi / 1.55 * scipy.constants.speed_of_light,
+            )
             integrals[i, j] = calculate_overlap(basis, E_i, H_i, basis, E_j, H_j)
 
     plt.imshow(np.real(integrals))
